#include "hardware.h"

void initMEM( void )
{
	int i;
	
	/* setup parallel flash */
	AVR32_SMC.cs[ 0 ].mode = 
	 	( AVR32_SMC_DBW_16_BITS << AVR32_SMC_DBW ) |
	 	( AVR32_SMC_BAT_BYTE_WRITE << AVR32_SMC_BAT ) |
	 	( AVR32_SMC_WRITE_MODE_NWE_CONTROLLED << AVR32_SMC_WRITE_MODE ) |
		( AVR32_SMC_READ_MODE_NRD_CONTROLLED << AVR32_SMC_READ_MODE );

	AVR32_SMC.cs[ 0 ].cycle =
	 	( 12 << AVR32_SMC_NRD_CYCLE ) |
	 	( 7 << AVR32_SMC_NWE_CYCLE );

	AVR32_SMC.cs[ 0 ].pulse =
	 	( 10 << AVR32_SMC_NCS_RD_PULSE ) |
	 	( 10 << AVR32_SMC_NRD_PULSE ) |
	 	( 4 << AVR32_SMC_NCS_WR_PULSE ) |
	 	( 4 << AVR32_SMC_NWE_PULSE ); 

	AVR32_SMC.cs[ 0 ].setup =
	 	( 1 << AVR32_SMC_NRD_SETUP ); 
	 
	 /* 
	  * SDRAM
	  * 4 Meg x 16 x 4 banks
	  * Refresh count 8K
	  * Row addressing 8k (A0-A12)
	  * Bank addressing 4 (BA0, BA1)
	  * Column addressing 512 (A0-A8)
	  * 143 MHz CL=3 / 133 MHz CL=2 Access 5.4ns, Setup 1.5ns, Hold 0.8ns
	  * PC100, PC133
	  * 64ms, 8,192 cycle refresh ---
	  */
	  
	/* set CS */
	AVR32_HMATRIX.sfr[ AVR32_EBI_HMATRIX_NR ] |= ( 1 << AVR32_EBI_SDRAM_CS );
	  
	/* 
	 * set timings 
	 *
	 * @ 75 MHz:
	 * 1 CK =  13,33 ns
	 * CAS = 2
	 * twr = 1 + 7ns = 2
	 * trc = 60ns = 5
	 * trp = 15ns = 2
	 * trcd = 15ns = 2
	 * tras = 37...120,000ns = 3
	 * txsr = 67ns = 6
	 */
	AVR32_SDRAMC.cr = 
		( 6 << AVR32_SDRAMC_TXSR ) |
		( 3 << AVR32_SDRAMC_TRAS ) |
		( 2 << AVR32_SDRAMC_TRCD ) |
		( 2 << AVR32_SDRAMC_TRP ) |
		( 5 << AVR32_SDRAMC_TRC ) |
		( 2 << AVR32_SDRAMC_TWR ) |
		( AVR32_SDRAMC_DBW_16_BITS << AVR32_SDRAMC_DBW ) |
		( AVR32_SDRAMC_CR_CAS_TWO_CYCLES << AVR32_SDRAMC_CAS ) |
		( AVR32_SDRAMC_NB_FOUR_BANKS << AVR32_SDRAMC_NB ) |
		( AVR32_SDRAMC_NR_13_ROW_BITS << AVR32_SDRAMC_NR ) |
		( AVR32_SDRAMC_NC_9_COLUMN_BITS << AVR32_SDRAMC_NC );
	  
	AVR32_SDRAMC.mdr = ( AVR32_SDRAMC_MD_SDRAM << AVR32_SDRAMC_MD );
	  
	/* wait 100 s */
	udelay( 100 );
	  
	/* precharge all */
	AVR32_SDRAMC.mr = ( AVR32_SDRAMC_BANKS_PRECHARGE << AVR32_SDRAMC_MODE );
	i = AVR32_SDRAMC.mr; /* wait a bit */
	*( ( volatile int * ) SDRAM_BASE ) = 0;
	  
	/* 2x auto refresh  */
	AVR32_SDRAMC.mr = ( AVR32_SDRAMC_AUTO_REFRESH << AVR32_SDRAMC_MODE );
	i = AVR32_SDRAMC.mr; /* wait a bit */
	for ( i = 0; i < 2; i++ ) {
		*( ( volatile int * ) SDRAM_BASE ) = 0;  	
	}
	  
	/* mode set */
	AVR32_SDRAMC.mr = ( AVR32_SDRAMC_LOAD_MODE << AVR32_SDRAMC_MODE );
	i = AVR32_SDRAMC.mr; /* wait a bit */
	*( ( volatile int * ) ( SDRAM_BASE + ( 2 << 4 ) ) ) = 0;  	
	
	/* switch to normal mode */
	i = AVR32_SDRAMC.mr; /* wait a bit */
	AVR32_SDRAMC.mr = ( AVR32_SDRAMC_NORMAL << AVR32_SDRAMC_MODE );
	i = AVR32_SDRAMC.mr; /* wait a bit */
	*( ( volatile int * ) SDRAM_BASE ) = 0;  	
	
	/* set refresh count */
	/* 64ms = 8192 cycles --> 586 @ 75 MHz */ 
	AVR32_SDRAMC.tr = 586;
}

void initClocks( void )
{
	/* startup PLL0 */
	AVR32_PM.icr = ( 1 << AVR32_PM_LOCK0 );
	AVR32_PM.pll0 =
		( 63 << AVR32_PM_PLLCOUNT ) |
		( ( 15 - 1 ) << AVR32_PM_PLLMUL ) |
		( ( 2 - 1 ) << AVR32_PM_PLLDIV ) |
		( 4 << AVR32_PM_PLLOPT ) | 
		AVR32_PM_PLLEN_MASK;
	
	/* wait till PLL0 has locked */
	while (! ( AVR32_PM.isr & AVR32_PM_LOCK0_MASK ) ) ;

	AVR32_PM.cpu_mask = 0x00000000; 
	AVR32_PM.hsb_mask = 0x000005DF; 
	AVR32_PM.pba_mask = 0x00007C15;
	AVR32_PM.pbb_mask = 0x000072C7;

	AVR32_PM.mcctrl = AVR32_PM_PLLSEL_MASK;

	/* 
	 * HSB, PBB / 2, PBA / 4
	 * HSB, PBB @ 75 MHz, PBA @ 37.5 MHz 
	 */
	AVR32_PM.cksel =
		AVR32_PM_PBBDIV_MASK |
		( ( 1 - 1 ) << AVR32_PM_PBBSEL ) |
		AVR32_PM_PBADIV_MASK |
		( ( 2 - 1 ) << AVR32_PM_PBASEL ) |
		AVR32_PM_HSBDIV_MASK |
		( ( 1 - 1 ) << AVR32_PM_HSBSEL );
}

void initPIO( void )
{
	/* setup pio mux */
	AVR32_PIOA.pdr = 0x0016FCCF;
	AVR32_PIOB.pdr = 0x00000000;
	AVR32_PIOC.pdr = 0xFDBFFFFF;
	AVR32_PIOD.pdr = 0x0003FFFF;
	AVR32_PIOE.pdr = 0x0007FFF7;
		
	/* peripheral a */
	AVR32_PIOA.asr = 0x0006FCCF;
	AVR32_PIOC.asr = 0x8037FFFF;
	AVR32_PIOD.asr = 0x00030383;
	
	/* peripheral b */
	AVR32_PIOA.bsr = 0x00100000;
	AVR32_PIOC.bsr = 0x7D880000;
	AVR32_PIOD.bsr = 0x0000FC7C;
	AVR32_PIOE.bsr = 0x0007FFF7;
			
	/* outputs */
	AVR32_PIOA.oer = 0x00090000;
	AVR32_PIOA.odr = 0xFFF6FFFF;

	AVR32_PIOB.odr = 0xFFFFFFFF;

	AVR32_PIOC.odr = 0xFFFFFFFF;

	AVR32_PIOD.odr = 0xFFFFFFFF;

	AVR32_PIOE.oer = 0x00080000;
	AVR32_PIOE.odr = 0xFFF7FFFF;
	
	/* gpio */
	AVR32_PIOA.per = 0x00090000; /* D3 LED + SYSTEM LED */
	AVR32_PIOC.per = 0x02000000; /* SD_DETECT */
	AVR32_PIOE.per = 0x00080001; /* D1 LED + SD_WP */

	/* inputs */
	AVR32_PIOA.pudr = 0xFFFFFFFF;
	AVR32_PIOB.pudr = 0xFFFFFFFF;
	AVR32_PIOC.pudr = 0xFFFFFFFF;
	AVR32_PIOD.pudr = 0xFFFFFFFF;
	AVR32_PIOE.pudr = 0xFFFFFFFF;	
}

void initHardware( void )
{
	initMEM();
	initINTC();
	initClocks();
	initPIO();
	initSerial();
	
	serialPutS( "*** RESET *** \r\n\n\n\n\n" );
	
	initUSB();
	
	while ( 1 ) {
		setLED( LED_A, 0 );
		udelay( 500000 );
		setLED( LED_A, 1 );
		udelay( 500000 );
	}
}


void setLED( int led, int status )
{
	switch ( led )
	{
	case LED_A:
		if ( status ) 
			AVR32_PIOA.codr = ( 1 << 19 );
		else 
			AVR32_PIOA.sodr = ( 1 << 19 );
		break;	
	case LED_B:
		if ( status ) 
			AVR32_PIOE.codr = ( 1 << 19 );
		else 
			AVR32_PIOE.sodr = ( 1 << 19 );
		break;	
	case LED_SYS:
		if ( status ) 
			AVR32_PIOA.codr = ( 1 << 16 );
		else 
			AVR32_PIOA.sodr = ( 1 << 16 );
		break;	
	}
}
