#include "hardware.h"

extern void evba;
extern void int0;
extern void int1;
extern void int2;
extern void int3;

int setEVBA( int );

#define g(x) { x, ( int ) ( &int ## x ) }

const struct {
	int prio;
	int handler;
} groupInfo[] = 
{
	g(0),	/* COUNT-COMPARE match, Performance Counter Overflow */
	g(2),	/* LCDC */
	g(3),	/* DMACA */
	g(1),	/* SPI 0 */
	g(0),	/* SPI 1 */
	g(0),	/* TWI */
	g(0),	/* USART0 */
	g(1),	/* USART1 */
	g(0),	/* USART2 */
	g(0),	/* USART3 */
	g(0),	/* SSC0 */
	g(0),	/* SSC1 */
	g(0),	/* SSC2 */
	g(1),	/* PIOA */
	g(1),	/* PIOB */
	g(1),	/* PIOC */
	g(1),	/* PIOD */
	g(1),	/* PIOE */
	g(0),	/* PSIF */
	g(0),	/* EICx */
	g(0),	/* PM */
	g(1),	/* RTC */
	g(0),	/* TC0x */
	g(0),	/* TC1x */
	g(0),	/* PWM */
	g(3),	/* MACB0 */
	g(3),	/* MACB1 */
	g(0),	/* PWM */
	g(2),	/* MCI */
	g(0),	/* AC97C */
	g(0),	/* ISI */
	g(3),	/* USBA */
	g(3)	/* EBI */
};

void initINTC( void )
{
	int i;
	
	/* setup EVBA */
	setEVBA( ( int ) &evba );
	
	/* setup INTx addresses with respect to evba */
	AVR32_INTC.ipr[ 0 ] = 0;
	
	/* setup group handlers and priority */
	for ( i = 0; i < sizeof( groupInfo ) / sizeof( groupInfo[ 0 ] ); i++ ) {
		AVR32_INTC.ipr[ i ] = 
			( groupInfo[ i ].prio << AVR32_INTC_IPR0_INTLEV ) |
			( groupInfo[ i ].handler - ( int ) &evba );
	}
	
	/* enable interrupts */
	intEnable();
}

void interrupt( int intlevel )
{
	int group = 0;
	
	group = 0;

	switch ( intlevel ) {
	case 0:
		group = AVR32_INTC.icr0;
		break;
	case 1:
		group = AVR32_INTC.icr1;
		break;
	case 2:
		group = AVR32_INTC.icr2;
		break;
	case 3:
		group = AVR32_INTC.icr3;
		break;
	}
	
	switch ( group ) {	
	/* LCDC */
	case 1:
		break;
	
	/* DMACA */
	case 2:
		break;
	
	/* SPI 0 */
	case 3:
		break;

	/* USART1 */
	case 7:
		break;
		
	/* PIOA */
	case 13:
		break;

	/* PIOB */
	case 14:
		break;

	/* PIOC */
	case 15:
		break;

	/* PIOD */
	case 16:
		break;

	/* PIOE */
	case 17:
		break;

	/* RTC */
	case 21:
		break;

	/* MACB0 */
	case 25:
		break;

	/* MACB1 */
	case 26:
		break;

	/* MCI */
	case 28:
		break;

	/* USBA */
	case 31:
		usbaInterrupt();
		break;
		
	/* EBI */
	case 32:
		break;
	}	
}
